/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amdigReg.c,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Globals:
 */

#if (DEBUG_ITIFG & 132)
const char  *dg_reg[] = { "modid ",
			  "modver",
			  "clkcon",
			  "ctcon ",
			  "excon ",
			  "extime",
			  "lutcon",
			  "lutsel",
			  "hoff  ",
			  "hact  ",
			  "voff  ",
			  "vact  ",
			  "trgcon",
			  "lstat ",
			  "swtrig" 
			};
#endif

/*
 * Prototypes:
 */

static u_short dg_read_mm (itifg_t*, u_short, u_short, u_short, u_short);
static u_char dg_read_ms (itifg_t*, u_short, u_short, u_char);
static int dg_write_mm (itifg_t*, u_short, u_short, u_short, u_short, u_short);
static int dg_write_ms (itifg_t*, u_short, u_short, u_char, u_char);

/*
 * General Read Handler for AM-DIG Registers
 */

/* 16 bit register access with mask and range checking */

static u_short
dg_read_mm (iti, offset, mask, min, max)
     
     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_short offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */

{
#if (DEBUG_ITIFG & 128)
  char   dg_reg_tmp[8];
#endif

  u_short tmp_val = 0;

  tmp_val = iti_readw (iti->v_mod_base + offset);

  tmp_val &= mask;

#if (0)
  if (tmp_val & ~mask) iti->errno = -EITI_MASK;

  if (tmp_val < min || tmp_val > max) iti->errno = -EITI_RANGE;
#endif

#if (DEBUG_ITIFG & 128)
  if (offset < DG_LUT16_OFFSET) {
    if (offset > DG_REG_SIZE) return (65535);
    iti_strcpy (dg_reg_tmp, dg_reg[offset]);
  } else {
    if (offset < DG_LUT8_OFFSET) {
      iti_strcpy (dg_reg_tmp, "lut16 ");
    } else {
      iti_strcpy (dg_reg_tmp, "lut8  ");
    }
  }
#endif
  ITI_PRINT7A("dg-readw %s, %#06X &(%#06X) >%#06X <%#06X\n",
	      dg_reg_tmp, tmp_val, mask, min, max);

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char
dg_read_ms (iti, offset, mask, shift)
     
     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_short offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char  shift;	/* number of bits to shift right */
  
{
  u_short tmp_val = 0;

  tmp_val = iti_readw (iti->v_mod_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

  ITI_PRINT7A("dg-readw %s, %#1X &(%#06X) >>%2d\n",
	      dg_reg[offset], tmp_val, mask, shift);

  return ((u_char)tmp_val);
}
  
/*
 * General Write Handler for AM-DIG Registers
 */

/* 16 bit register access with mask and range checking */

static int
dg_write_mm (iti, offset, mask, min, max, value)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_short offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_short min;	/* minimal value for range checking */
     u_short max;	/* maximal value for range checking */
     u_short value;	/* value to set to */

{
#if (DEBUG_ITIFG & 128)
  char	 dg_reg_tmp[8];
#endif

  u_short tmp_val = 0;

  if (value & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }

  if (value < min || value > max) {
    iti->errno = -EITI_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

#if (DEBUG_ITIFG & 128)
  if (offset < DG_LUT16_OFFSET) {
    if (offset > DG_REG_SIZE) {
      iti->errno = -EITI_NOSUPPORT;  
      return (ERROR);
    }
    iti_strcpy (dg_reg_tmp, dg_reg[offset]);
  } else {
    if (offset < DG_LUT8_OFFSET) {
      iti_strcpy (dg_reg_tmp, "lut16 ");
    } else {
      iti_strcpy (dg_reg_tmp, "lut8  ");
    }
  }
#endif
  ITI_PRINT7A("dg-writew %s, %#06X &(%#06X) >%#06X <%#06X\n",
	      dg_reg_tmp, tmp_val, mask, min, max);


  iti_writew (tmp_val, iti->v_mod_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

static int
dg_write_ms (iti, offset, mask, shift, value)

     itifg_t *iti;	/* number of IC-PCI board attached to the computer */
     u_short offset;	/* offset to register base address pointer */
     u_short mask;	/* read mask for register */
     u_char  shift;	/* number of bits to shift right */
     u_char  value;	/* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;

  if (offset > DG_REG_SIZE) {
    iti->errno = -EITI_NOSUPPORT;    
    return (ERROR);
  }

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    iti->errno = -EITI_MASK;
    return (ERROR);
  }

  read_val = iti_readw (iti->v_mod_base + offset);

  read_val &= ~mask;
  write_val = read_val | tmp_val;

  ITI_PRINT7A("dg-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
	      dg_reg[offset], read_val, write_val, tmp_val, mask, shift);

  iti_writew (write_val, iti->v_mod_base + offset);

  return (OK);
}
  
/*
 * Special Read Handler for AM-DIG Registers
 */

u_char
dg_get_rev (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_MODID_OFFSET,
		      DG_REV_MASK,
		      DG_REV_SHIFT));
}

u_char
dg_get_modver (itifg_t *iti) {

  u_short tmp_val = 0;
  
  tmp_val = dg_read_mm (iti,
			DG_MODVER_OFFSET,
			DG_MODVER_MASK,
			DG_MODVER_MIN,
			DG_MODVER_MAX);

#if(0)
  if (tmp_val > 4) iti->errno = -EITI_RANGE;
#endif

  return ((u_char)tmp_val);
}

u_char
dg_get_fsel (itifg_t *iti) {

  u_char tmp_val = 0;
  
  tmp_val = dg_read_ms (iti,
			DG_CLKCON_OFFSET,
			DG_FSEL_MASK,
			DG_FSEL_SHIFT);

#if(0)
  switch (tmp_val) {
  case 2:
  case 12:
  case 14:
  case 15:
    iti->errno = -EITI_RANGE;	
    break;
  }
#endif
  
  return (tmp_val);
}

u_char
dg_get_clkdiv (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CLKCON_OFFSET,
		      DG_CLKDIV_MASK,
		      DG_CLKDIV_SHIFT));
}

u_char
dg_get_xillen (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CLKCON_OFFSET,
		      DG_XILLEN_MASK,
		      DG_XILLEN_SHIFT));
}

u_char
dg_get_mc (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CLKCON_OFFSET,
		      DG_MC_MASK,
		      DG_MC_SHIFT));
}

u_char
dg_get_pclkpol (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_PCLKPOL_MASK,
		      DG_PCLKPOL_SHIFT));
}

u_char
dg_get_lenpol (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_LENPOL_MASK,
		      DG_LENPOL_SHIFT));
}

u_char
dg_get_fenpol (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_FENPOL_MASK,
		      DG_FENPOL_SHIFT));
}

u_char
dg_get_fldpol (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_FLDPOL_MASK,
		      DG_FLDPOL_SHIFT));
}

u_char
dg_get_smode (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_SMODE_MASK,
		      DG_SMODE_SHIFT));
}

u_char
dg_get_ilmode (itifg_t *iti) {
  
  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_ILMODE_MASK,
		      DG_ILMODE_SHIFT));
}

u_char
dg_get_lensm (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_LENSM_MASK,
		      DG_LENSM_SHIFT));
}

u_char
dg_get_nolmiss (itifg_t *iti) {

  u_char tmp_val;

  tmp_val =  (dg_read_ms (iti,
			  DG_CTCON_OFFSET,
			  DG_NOLMISS_MASK,
			  DG_NOLMISS_SHIFT));

#if(0)
  if (tmp_val != DG_ENABLE) iti->errno = -EITI_NOSUPPORT;
#endif

  return (tmp_val);
}

u_char
dg_get_lvar (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_CTCON_OFFSET,
		      DG_LVAR_MASK,
		      DG_LVAR_SHIFT));
}

u_char
dg_get_exclk (itifg_t *iti) {

  u_char tmp_val = 0;

  tmp_val = dg_read_ms (iti,
			DG_EXCON_OFFSET,
			DG_EXCLK_MASK,
			DG_EXCLK_SHIFT);
  
#if(0)
  if (tmp_val > 5) iti->error = -EITI_RANGE;
#endif

  return (tmp_val);
}

u_char
dg_get_extmd (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_EXCON_OFFSET,
		      DG_EXTMD_MASK,
		      DG_EXTMD_SHIFT));
}

u_char
dg_get_exsen (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_EXCON_OFFSET,
		      DG_EXSEN_MASK,
		      DG_EXSEN_SHIFT));
}

u_char
dg_get_exspol (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_EXCON_OFFSET,
		      DG_EXSPOL_MASK,
		      DG_EXSPOL_SHIFT));
}

u_char
dg_get_prien (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_EXCON_OFFSET,
		      DG_PRIEN_MASK,
		      DG_PRIEN_SHIFT));
}

u_char
dg_get_prip (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_EXCON_OFFSET,
		      DG_PRIP_MASK,
		      DG_PRIP_SHIFT));
}

u_short
dg_get_extime (itifg_t *iti) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (iti,
			DG_EXTIME_OFFSET,
			DG_EXTIME_MASK,
			DG_EXTIME_MIN,
			DG_EXTIME_MAX);
  
  return (tmp_val + 3);

}

u_char
dg_get_psize (itifg_t *iti) {

  u_char tmp_val = 0; 
  tmp_val = dg_read_ms (iti,
			DG_LUTCON_OFFSET,
			DG_PSIZE_MASK,
			DG_PSIZE_SHIFT);

  switch (tmp_val)
    {
    case DG_PSIZE_8:
      return 8;
    case DG_PSIZE_10:
      return 10;
    case DG_PSIZE_12:
      return 12;
    case DG_PSIZE_16:
      return 16;
    default:
      iti->errno = -EITI_MASK;
      return ERROR;
    }
  
  iti->errno = -EITI_MASK;
  return ERROR;
}
u_char
dg_get_ilpixel (itifg_t *iti) {
  
  return (dg_read_ms (iti,
		      DG_LUTCON_OFFSET,
		      DG_ILPIXEL_MASK,
		      DG_ILPIXEL_SHIFT));
}

u_char
dg_get_muxa (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_LUTCON_OFFSET,
		      DG_MUXA_MASK,
		      DG_MUXA_SHIFT));
}

u_char
dg_get_muxb (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_LUTCON_OFFSET,
		      DG_MUXB_MASK,
		      DG_MUXB_SHIFT));
}

u_char
dg_get_muxc (itifg_t *iti) {

#if(0)
  if (dg_get_rev (iti) < 2 || dg_get_modver (iti) == DG_MODVER_24_TTL)
    iti->errno = -EITI_NOSUPPORT;
#endif

  return (dg_read_ms (iti,
		      DG_LUTCON_OFFSET,
		      DG_MUXC_MASK,
		      DG_MUXC_SHIFT));
}

u_char
dg_get_lutpg (itifg_t *iti) {

  u_char tmp_val;

#if(0)
  u_char mod_ver;

  mod_ver = dg_get_modver (iti);
  if (modver == DG_MODVER_8_TTL || modver == DG_MODVER_8_RS422) {
    iti->errno = -EITI_NOSUPPORT;
#endif

  tmp_val = dg_read_ms (iti,
			DG_LUTSEL_OFFSET,
			DG_LUTPG_MASK,
			DG_LUTPG_SHIFT);

  return (tmp_val);
}

u_char
dg_get_olutpg (itifg_t *iti) {

#if(0)
  if (dg_get_modver (iti) != DG_MODVER_24_TTL)
    iti->errno = -EITI_NOSUPPORT;
#endif


  return (dg_read_ms (iti,
		      DG_LUTSEL_OFFSET,
		      DG_OLUTPG_MASK,
		      DG_OLUTPG_SHIFT));
}

u_short
dg_get_hoff (itifg_t *iti) {

  return (dg_read_mm (iti,
		      DG_HOFF_OFFSET,
		      DG_HOFF_MASK,
		      DG_HOFF_MIN,
		      DG_HOFF_MAX));
}

u_short
dg_get_hact (itifg_t *iti) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (iti,
			DG_HACT_OFFSET,
			DG_HACT_MASK,
			DG_HACT_MIN,
			DG_HACT_MAX);
  
  return (tmp_val + 4);
}

u_short
dg_get_voff (itifg_t *iti) {

  return (dg_read_mm (iti,
		      DG_VOFF_OFFSET,
		      DG_VOFF_MASK,
		      DG_VOFF_MIN,
		      DG_VOFF_MAX));
}

u_short
dg_get_vact (itifg_t *iti) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (iti,
		      DG_VACT_OFFSET,
		      DG_VACT_MASK,
		      DG_VACT_MIN,
		      DG_VACT_MAX);

  return (tmp_val + 1);
}

u_char
dg_get_trigen (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_TRIGCON_OFFSET,
		      DG_TRIGEN_MASK,
		      DG_TRIGEN_SHIFT));
}

u_char
dg_get_trigsrc (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_TRIGCON_OFFSET,
		      DG_TRIGSRC_MASK,
		      DG_TRIGSRC_SHIFT));
}

u_char
dg_get_trigcy (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_TRIGCON_OFFSET,
		      DG_TRIGCY_MASK,
		      DG_TRIGCY_SHIFT));
}

u_char
dg_get_vb (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_TRIGCON_OFFSET,
		      DG_VBSTAT_MASK,
		      DG_VBSTAT_SHIFT));
}

u_char
dg_get_fld (itifg_t *iti) {

  return (dg_read_ms (iti,
		      DG_TRIGCON_OFFSET,
		      DG_FLD_MASK,
		      DG_FLD_SHIFT));
}

u_short
dg_get_lstat (itifg_t *iti) {

  return (dg_read_mm (iti,
		      DG_LSTAT_OFFSET,
		      DG_LSTAT_MASK,
		      DG_LSTAT_MIN,
		      DG_LSTAT_MAX));
}

u_char
dg_get_lut8 (itifg_t *iti, struct dg_lut8_t *get_lut) {

  int count;

#if(0)
  if (dg_get_modver (iti) != DG_MODVER_24_TTL)
    iti->errno = -EITI_NOSUPPORT;
  if (get_lut == NULL) iti->errno = -EITI_POINTER;
#endif

  for (count = 0; count < DG_LUT8_SIZE; count++) {

    get_lut->lutdata[count] = dg_read_mm (iti,
					  DG_LUT8_OFFSET + count,
					  DG_LUT8_MASK,
					  DG_LUT8_MIN,
					  DG_LUT8_MAX);
  }

  return (OK);
}
			
/*
 * Special Write Handler for AM-DIG Registers
 */

int
dg_set_fsel (itifg_t *iti, u_char value) {

    switch (value) {
    case 2:
    case 12:
    case 14:
    case 15:
      iti->errno = -EITI_RANGE;
      return (ERROR);
      break;
    }	

    return (dg_write_ms (iti,
			 DG_CLKCON_OFFSET,
			 DG_FSEL_MASK,
			 DG_FSEL_SHIFT,
			 value));
}

int
dg_set_clkdiv (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CLKCON_OFFSET,
		       DG_CLKDIV_MASK,
		       DG_CLKDIV_SHIFT,
		       value));
}

int
dg_set_xillen (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CLKCON_OFFSET,
		       DG_XILLEN_MASK,
		       DG_XILLEN_SHIFT,
		       value));
}

int
dg_set_mc (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CLKCON_OFFSET,
		       DG_MC_MASK,
		       DG_MC_SHIFT,
		       value));
}

int
dg_set_pclkpol (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_PCLKPOL_MASK,
		       DG_PCLKPOL_SHIFT,
		       value));
}

int
dg_set_lenpol (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_LENPOL_MASK,
		       DG_LENPOL_SHIFT,
		       value));
}

int
dg_set_fenpol (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_FENPOL_MASK,
		       DG_FENPOL_SHIFT,
		       value));
}

int
dg_set_fldpol (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_FLDPOL_MASK,
		       DG_FLDPOL_SHIFT,
		       value));
}

int
dg_set_smode (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_SMODE_MASK,
		       DG_SMODE_SHIFT,
		       value));
}

int
dg_set_ilmode (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_ILMODE_MASK,
		       DG_ILMODE_SHIFT,
		       value));
}

int
dg_set_lensm (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_LENSM_MASK,
		       DG_LENSM_SHIFT,
		       value));
}

int
dg_set_nolmiss (itifg_t *iti, u_char value) {

#if (0)
  if (value != DG_ENABLE) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
#endif

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_NOLMISS_MASK,
		       DG_NOLMISS_SHIFT,
		       value));
}

int
dg_set_lvar (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_CTCON_OFFSET,
		       DG_LVAR_MASK,
		       DG_LVAR_SHIFT,
		       value));
}

int
dg_set_exclk (itifg_t *iti, u_char value) {

  if (value > 5) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_EXCLK_MASK,
		       DG_EXCLK_SHIFT,
		       value));
}

int
dg_set_extmd (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_EXTMD_MASK,
		       DG_EXTMD_SHIFT,
		       value));
}

int
dg_set_exsen (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_EXSEN_MASK,
		       DG_EXSEN_SHIFT,
		       value));
}

int
dg_set_exspol (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_EXSPOL_MASK,
		       DG_EXSPOL_SHIFT,
		       value));
}

int
dg_set_prien (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_PRIEN_MASK,
		       DG_PRIEN_SHIFT,
		       value));
}

int
dg_set_prip (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_EXCON_OFFSET,
		       DG_PRIP_MASK,
		       DG_PRIP_SHIFT,
		       value));
}

int
dg_set_extime (itifg_t *iti, u_short value) {

  if (value < 3 || value > 4098) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  return (dg_write_mm (iti,
		       DG_EXTIME_OFFSET,
		       DG_EXTIME_MASK,
		       DG_EXTIME_MIN,
		       DG_EXTIME_MAX,
		       value - 3));
}

int
dg_set_psize (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_LUTCON_OFFSET,
		       DG_PSIZE_MASK,
		       DG_PSIZE_SHIFT,
		       value));
}

int
dg_set_ilpixel (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_LUTCON_OFFSET,
		       DG_ILPIXEL_MASK,
		       DG_ILPIXEL_SHIFT,
		       value));
}

int
dg_set_muxa (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_LUTCON_OFFSET,
		       DG_MUXA_MASK,
		       DG_MUXA_SHIFT,
		       value));
}

int
dg_set_muxb (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_LUTCON_OFFSET,
		       DG_MUXB_MASK,
		       DG_MUXB_SHIFT,
		       value));
}

int
dg_set_muxc (itifg_t *iti, u_char value) {


  if (dg_get_rev (iti) < 2 || dg_get_modver (iti) == DG_MODVER_24_TTL) {
    if (value == DG_ENABLE) {
      iti->errno = -EITI_NOSUPPORT;
      return (ERROR);
    }
  }

  return (dg_write_ms (iti,
		       DG_LUTCON_OFFSET,
		       DG_MUXC_MASK,
		       DG_MUXC_SHIFT,
		       value));
}

int
dg_set_lutpg (itifg_t *iti, u_char value) {

  u_char mod_ver;

  mod_ver = dg_get_modver (iti);
  if (mod_ver == DG_MODVER_8_TTL || mod_ver == DG_MODVER_8_RS422) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }

  return (dg_write_ms (iti,
		       DG_LUTSEL_OFFSET,
		       DG_LUTPG_MASK,
		       DG_LUTPG_SHIFT,
		       value));
}

int
dg_set_olutpg (itifg_t *iti, u_char value) {

  if (dg_get_modver (iti) != DG_MODVER_24_TTL) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }

  return (dg_write_ms (iti,
		       DG_LUTSEL_OFFSET,
		       DG_OLUTPG_MASK,
		       DG_OLUTPG_SHIFT,
		       value));
}

int
dg_set_hoff (itifg_t *iti, u_short value) {

  return (dg_write_mm (iti,
		       DG_HOFF_OFFSET,
		       DG_HOFF_MASK,
		       DG_HOFF_MIN,
		       DG_HOFF_MAX,
		       value));
}

int
dg_set_hact (itifg_t *iti, u_short value) {

  return (dg_write_mm (iti,
		       DG_HACT_OFFSET,
		       DG_HACT_MASK,
		       DG_HACT_MIN,
		       DG_HACT_MAX,
		       (value - 4)));
}

int
dg_set_voff (itifg_t *iti, u_short value) {

  return (dg_write_mm (iti,
		       DG_VOFF_OFFSET,
		       DG_VOFF_MASK,
		       DG_VOFF_MIN,
		       DG_VOFF_MAX,
		       value));
}

int
dg_set_vact (itifg_t *iti, u_short value) {

  return (dg_write_mm (iti,
		       DG_VACT_OFFSET,
		       DG_VACT_MASK,
		       DG_VACT_MIN,
		       DG_VACT_MAX,
		       value - 1));
}

int
dg_set_trigen (itifg_t *iti, u_char value) {

  if (value == DG_ENABLE) {
    if (dg_get_trigcy (iti) == DG_DISABLE) {
      iti->errno = -EITI_BUSY;
      return (ERROR);
    }
  }

  return (dg_write_ms (iti,
		       DG_TRIGCON_OFFSET,
		       DG_TRIGEN_MASK,
		       DG_TRIGEN_SHIFT,
		       value));
}

int
dg_set_trigsrc (itifg_t *iti, u_char value) {

  return (dg_write_ms (iti,
		       DG_TRIGCON_OFFSET,
		       DG_TRIGSRC_MASK,
		       DG_TRIGSRC_SHIFT,
		       value));
}

int
dg_set_lut8 (itifg_t *iti, struct dg_lut8_t *set_lut) {

  int count;

  if (dg_get_modver (iti) != DG_MODVER_24_TTL) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }

  if (set_lut == NULL) {
    iti->errno = -EITI_POINTER;
    return (ERROR);
  }

  for (count = 0; count < DG_LUT8_SIZE; count++) {

    if (dg_write_mm (iti,
		     DG_LUT8_OFFSET + count,
		     DG_LUT8_MASK,
		     DG_LUT8_MIN,
		     DG_LUT8_MAX,
		     set_lut->lutdata[count]) < 0) return (ERROR);
  }
  
  return (OK);
}

/*
 * Special Initialization Handler for AM-DIG Registers
 */

int
dg_init_clkcon (itifg_t *iti) {

  return (dg_write_mm (iti,
		       DG_CLKCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_ctcon (itifg_t *iti) {
    
  return (dg_write_mm (iti,
		       DG_CTCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_excon (itifg_t *iti) {
    
    return (dg_write_mm (iti,
			 DG_EXCON_OFFSET,
			 DG_NO_MASK,
			 DG_NO_MIN,
			 DG_NO_MAX,
			 DG_RES_INIT));
}

int
dg_init_lutcon (itifg_t *iti) {
    
    return (dg_write_mm (iti,
			 DG_LUTCON_OFFSET,
			 DG_NO_MASK,
			 DG_NO_MIN,
			 DG_NO_MAX,
			 DG_RES_INIT));
}

int
dg_init_lutsel (itifg_t *iti) {

  return (dg_write_mm (iti,
		       DG_LUTSEL_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_trigcon (itifg_t *iti) {

  return (dg_write_mm (iti,
		       DG_TRIGCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

/* 
 * Miscellanous functions
 */
 
/* trigger per software */

int
dg_sw_trig (itifg_t *iti) {

  return (dg_write_mm (iti,
		       DG_SWTRIG_OFFSET,
		       DG_SWTRIG_MASK,
		       DG_SWTRIG_MIN,
		       DG_SWTRIG_MAX,
		       DG_SWTRIG));
}

int
dg_lin_lut8 (itifg_t *iti) {

  int	count = 0,
    	page = 0;

  struct dg_lut8_t lin_lut;  

  for (count = 0; count < DG_LUT8_SIZE; count++)
    lin_lut.lutdata[count] = count;

  if (dg_set_olutpg (iti, 0) < 0) return (ERROR);
  if (dg_set_lut8 (iti, &lin_lut) < 0) return (ERROR);

  iti_memset (&lin_lut, 0x00, sizeof(struct dg_lut8_t));

  for (page = 1; page < DG_OLUTPG_MAX; page++) {

    if (dg_set_olutpg (iti, page) < 0) return (ERROR);
    if (dg_set_lut8 (iti, &lin_lut) < 0) return (ERROR);
  }

  if (dg_set_olutpg (iti, 0) < 0) return (ERROR);

  return (OK);
}

#if (DEBUG_ITIFG & 4)
/* make register dump */
void
dg_dump_regs (itifg_t *iti) {
    
  int count;

  for (count = 0; count < DG_REG_SIZE; count++) {
    ITI_PRINT2A("%s %#04x\n", dg_reg[count],
		dg_read_mm (iti, count, DG_NO_MASK, DG_NO_MIN, DG_NO_MAX));
  }
}
#endif

