/*		 IIT-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: pcdigReg.c,v 7.0 2000/10/25 09:07:58 cvs Exp $
 *
 */

/*
 * Defines:
 */

#if (DEBUG_ITIFG & 4)
#  define DEBUG_ITIFG_CLK
#endif
#ifdef DEBUG_ITIFG_CLK
#  define PCD_PRINTR(fmt,args...) iti_printra (fmt, ## args)
#else
#  define PCD_PRINTR(fmt,args...)
#endif

#define READ_HANDLER_BIT(reg_set, reg_name, subfield) \
	inline u_char \
	pcd_get_ ## subfield (itifg_t *iti) \
	{ \
	  pcd_ ## reg_set ## _ ## reg_name ## _t tmp_val; \
	  tmp_val.all = \
	  iti_readl (&iti->pcd_ ## reg_set ## _reg-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define READ_HANDLER_FIELD(reg_set, reg_name, subfield) \
	inline u_char \
	pcd_get_ ## subfield (itifg_t *iti) \
	{ \
	  pcd_ ## reg_set ## _ ## reg_name ## _t tmp_val; \
	  tmp_val.all = \
	  iti_readl (&iti->pcd_ ## reg_set ## _reg-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define WRITE_HANDLER_BIT(reg_set, reg_name, subfield) \
	inline int \
	pcd_set_ ## subfield (itifg_t *iti, u_char bit_value) \
	{ \
	  pcd_ ## reg_set ## _ ## reg_name ## _t tmp_val; \
	  tmp_val.all = \
	  iti_readl (&iti->pcd_ ## reg_set ## _reg-> ## reg_name ## .all); \
          tmp_val.b. ## subfield = bit_value; \
	  iti_writel (tmp_val.all, \
	      	      &iti->pcd_ ## reg_set ## _reg-> ## reg_name ## .all); \
	  return OK; \
        }
				 
/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Globals:
 */

#ifdef DEBUG_ITIFG
const char *pcd_bi_reg_name[] =
{
  "bmcon  ",
  "bmlut  ",
  "error  ",
  "aqstart",
  "aqadr  ",
  "intadr ",
  "zoom   ",
  "segsize",
  "octstrt",
  "swcount",
  "error  ",
  "intcon1",
  "intstat",
};
const char *pcd_ca_reg_name[] =
{
  "ctcon  ",
  "outcon ",
  "exscon ",
  "exstime",
  "acqcon ",
  "error  ",
  "hoff   ",
  "hact   ",
  "voff   ",
  "vact   ",
  "cclkcon",
  "lvact  ",
  "lutcon0",
  "lutcon1",
  "trigcon",
  "trigsta",
  "error  ",
  "error  ",
  "intcon0",
  "intstat",
  "trigdiv",
  "pvact  ",
  "pmidpnt",
  "decim  ",
  "sform  ",
  "error  ",
  "portcon",
  "outport",
  "inport ",
};
#endif /* DEBUG_ITIFG_REG */

/*
 * Prototypes:
 */

static int
pcd_set_syndat (itifg_t*, u_char);
static int
pcd_set_synclka (itifg_t*, u_char);
static int
pcd_set_synclkb (itifg_t*, u_char);

/*
 * Special Read Handler for PCDig Control Registers
 */

READ_HANDLER_BIT(bi,bmcon,bmenb)
READ_HANDLER_FIELD(bi,bmcon,bmstep)
READ_HANDLER_BIT(bi,bmlut,octacc)
READ_HANDLER_BIT(bi,bmlut,dmaacc)
READ_HANDLER_BIT(bi,zoom,octswap)
READ_HANDLER_FIELD(bi,zoom,pad)
READ_HANDLER_BIT(bi,zoom,clip)
READ_HANDLER_BIT(bi,intcon1,adrien)
READ_HANDLER_BIT(bi,intcon1,bmien)
READ_HANDLER_BIT(bi,intcon1,swctien)
READ_HANDLER_BIT(bi,intstat1,adrint)
READ_HANDLER_BIT(bi,intstat1,bmint)
READ_HANDLER_BIT(bi,intstat1,swctint)
  
u_long
pcd_get_aqstart (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_bi_reg->aqstart) & PCD_AQSTART_MASK)
	  << PCD_AQSTART_SHIFT);
}  

u_long
pcd_get_aqadr (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_bi_reg->aqadr) & PCD_AQADR_MASK)
	  << PCD_AQADR_SHIFT);
}  

u_long
pcd_get_intadr (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_bi_reg->intadr) & PCD_INTADR_MASK)
	  << PCD_INTADR_SHIFT);
}  

u_long
pcd_get_segsize (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_bi_reg->segsize) & PCD_SEGSIZE_MASK) + 1);
}  

u_short
pcd_get_octstart (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_bi_reg->octstart) & PCD_OCTSTART_MASK);
}  

u_long
pcd_get_swcount (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_bi_reg->swcount) & PCD_SWCOUNT_MASK);
}  

READ_HANDLER_BIT(ca,ctcon,lenenb)
READ_HANDLER_BIT(ca,ctcon,lensync)
READ_HANDLER_BIT(ca,ctcon,lenpol)
READ_HANDLER_BIT(ca,ctcon,fenenb)
READ_HANDLER_BIT(ca,ctcon,fenpol)
READ_HANDLER_BIT(ca,ctcon,multenb)
READ_HANDLER_BIT(ca,ctcon,multpol)
READ_HANDLER_FIELD(ca,ctcon,multmd)
READ_HANDLER_BIT(ca,ctcon,pclkpol)
READ_HANDLER_BIT(ca,outcon,driveext)
READ_HANDLER_BIT(ca,outcon,useext)
READ_HANDLER_BIT(ca,outcon,restime)
READ_HANDLER_BIT(ca,outcon,respol)
READ_HANDLER_BIT(ca,outcon,cam3enb)
READ_HANDLER_BIT(ca,outcon,exsenb)
READ_HANDLER_BIT(ca,outcon,exspol)
READ_HANDLER_FIELD(ca,outcon,exsmd)
READ_HANDLER_BIT(ca,outcon,prienb)
READ_HANDLER_BIT(ca,outcon,pripol)
READ_HANDLER_FIELD(ca,outcon,cammd)
READ_HANDLER_BIT(ca,outcon,cclkenb)
READ_HANDLER_BIT(ca,outcon,cclkpol)
READ_HANDLER_FIELD(ca,exscon,exsclk)
READ_HANDLER_FIELD(ca,exscon,exssel)
READ_HANDLER_FIELD(ca,exscon,prisel)
READ_HANDLER_FIELD(ca,exscon,prihold)
READ_HANDLER_FIELD(ca,acqcon,stmode)
READ_HANDLER_FIELD(ca,acqcon,etmode)
READ_HANDLER_FIELD(ca,acqcon,acqmd)
READ_HANDLER_FIELD(ca,acqcon,fldsel)
READ_HANDLER_BIT(ca,acqcon,newaq)
READ_HANDLER_BIT(ca,acqcon,gstat)
READ_HANDLER_BIT(ca,acqcon,fstrt)
READ_HANDLER_BIT(ca,cclkcon,synmuxa)
READ_HANDLER_BIT(ca,cclkcon,synmuxb)
READ_HANDLER_BIT(ca,lutcon0,lutacc)
READ_HANDLER_FIELD(ca,lutcon0,lut0pg)
READ_HANDLER_FIELD(ca,lutcon1,lut1pg)
READ_HANDLER_BIT(ca,trigcon,trig0pol)
READ_HANDLER_FIELD(ca,trigcon,trig0src)
READ_HANDLER_BIT(ca,trigcon,trig0gd)
READ_HANDLER_BIT(ca,trigcon,trig1pol)
READ_HANDLER_FIELD(ca,trigcon,trig1src)
READ_HANDLER_BIT(ca,trigcon,trig1gd)
READ_HANDLER_BIT(ca,trigstat,trig0diff)
READ_HANDLER_BIT(ca,trigstat,trig0ttl)
READ_HANDLER_BIT(ca,trigstat,trig0opto)
READ_HANDLER_BIT(ca,trigstat,trig0cyc)
READ_HANDLER_BIT(ca,trigstat,trig1diff)
READ_HANDLER_BIT(ca,trigstat,trig1ttl)
READ_HANDLER_BIT(ca,trigstat,trig1opto)
READ_HANDLER_BIT(ca,trigstat,trig1cyc)
READ_HANDLER_BIT(ca,trigstat,camlen)
READ_HANDLER_BIT(ca,trigstat,camfen)
READ_HANDLER_BIT(ca,trigstat,cammult)
READ_HANDLER_BIT(ca,intcon0,vbien)
READ_HANDLER_BIT(ca,intcon0,hbien)
READ_HANDLER_BIT(ca,intcon0,trig0ien)
READ_HANDLER_BIT(ca,intcon0,trig1ien)
READ_HANDLER_BIT(ca,intcon0,soaqien)
READ_HANDLER_BIT(ca,intcon0,eoaqien)
READ_HANDLER_BIT(ca,intcon0,pioien)
READ_HANDLER_BIT(ca,intstat0,vbint)
READ_HANDLER_BIT(ca,intstat0,hbint)
READ_HANDLER_BIT(ca,intstat0,trig0int)
READ_HANDLER_BIT(ca,intstat0,trig1int)
READ_HANDLER_BIT(ca,intstat0,soaqint)
READ_HANDLER_BIT(ca,intstat0,eoaqint)
READ_HANDLER_BIT(ca,intstat0,pioint)
READ_HANDLER_FIELD(ca,decim,hdec)
READ_HANDLER_FIELD(ca,decim,vdec)
READ_HANDLER_FIELD(ca,sform,sform)
READ_HANDLER_BIT(ca,portcon,outstrb)
READ_HANDLER_BIT(ca,portcon,inregenb)
READ_HANDLER_BIT(ca,portcon,instrbpol)
READ_HANDLER_BIT(ca,portcon,inregclr)
READ_HANDLER_BIT(ca,portcon,inpintpol)
READ_HANDLER_BIT(ca,portcon,inprint)
READ_HANDLER_BIT(ca,portcon,inprstrb)

u_short
pcd_get_emidpnt (itifg_t *iti)
{
  pcd_ca_exscon_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  return (tmp_val.b.emidpnt << PCD_EMIDPNT_SHIFT);
}  

u_long
pcd_get_exstime (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_ca_reg->exstime) & PCD_EXSTIME_MASK) + 2);
}  

u_char
pcd_get_fcnt (itifg_t *iti)
{
  pcd_ca_acqcon_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  return (++tmp_val.b.fcnt);
}  

u_short
pcd_get_hoff (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->hoff) & PCD_HOFF_MASK);
}  

u_long
pcd_get_hact (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_ca_reg->hact) & PCD_HACT_MASK) + 1);
}  

int
pcd_get_lut0psz (itifg_t *iti)
{
  pcd_ca_lutcon0_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon0.all);

  switch (tmp_val.b.lut0psz)
    {
    case PCD_PSIZE_8:
      return 8;
    case PCD_PSIZE_10:
      return 10;
    case PCD_PSIZE_12:
      return 12;
    case PCD_PSIZE_14:
      return 14;
    case PCD_PSIZE_16:
      return 16;
    default:
      iti->errno = -EITI_MASK;
      return ERROR;
    }
     
  iti->errno = -EITI_MASK;
  return ERROR;
}

int
pcd_get_lut1psz (itifg_t *iti)
{
  pcd_ca_lutcon1_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon1.all);

  switch (tmp_val.b.lut1psz)
    {
    case PCD_PSIZE_8:
      return 8;
    case PCD_PSIZE_10:
      return 10;
    case PCD_PSIZE_12:
      return 12;
    case PCD_PSIZE_14:
      return 14;
    case PCD_PSIZE_16:
      return 16;
    default:
      iti->errno = -EITI_MASK;
      return ERROR;
    }
     
  iti->errno = -EITI_MASK;
  return ERROR;
}

u_short
pcd_get_voff (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->voff) & PCD_VOFF_MASK);
}  

u_long
pcd_get_vact (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_ca_reg->vact) & PCD_VACT_MASK) + 1);
}  

u_short
pcd_get_lvact (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->lvact) & PCD_LVACT_MASK);
}  

u_short
pcd_get_divcon (itifg_t *iti)
{
  pcd_ca_trigdiv_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigdiv.all);
  return (++tmp_val.b.divcon);
}  

u_short
pcd_get_trigdiv (itifg_t *iti)
{
  pcd_ca_trigdiv_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigdiv.all);
  return (++tmp_val.b.trigdiv);
}  

u_short
pcd_get_pvact (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->pvact) & PCD_PVACT_MASK);
}  

u_short
pcd_get_pmidpnt (itifg_t *iti)
{
  return ((iti_readl (&iti->pcd_ca_reg->pmidpnt) & PCD_PMIDPNT_MASK)
	  << PCD_PMIDPNT_SHIFT);
}  

u_char
pcd_get_outport (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->outport));
}  

u_char
pcd_get_inport (itifg_t *iti)
{
  return (iti_readl (&iti->pcd_ca_reg->inport));
}  

READ_HANDLER_BIT(fp,fpgacon,fpgaprg)
READ_HANDLER_BIT(fp,fpgacon,fpgardy)
READ_HANDLER_BIT(fp,fpgacon,fpgadone)
READ_HANDLER_FIELD(fp,fpgacon,brdid)

/*
 * Special Write Handler for PCDig Control Registers
 */

WRITE_HANDLER_BIT(bi,bmcon,bmenb)
WRITE_HANDLER_BIT(bi,bmlut,octacc)
WRITE_HANDLER_BIT(bi,bmlut,dmaacc)
WRITE_HANDLER_BIT(bi,zoom,octswap)
WRITE_HANDLER_BIT(bi,zoom,clip)
WRITE_HANDLER_BIT(bi,intcon1,adrien)
WRITE_HANDLER_BIT(bi,intcon1,bmien)
WRITE_HANDLER_BIT(bi,intcon1,swctien)

int
pcd_set_bmstep (itifg_t *iti, u_char value)
{
  pcd_bi_bmcon_t tmp_val;

  switch (value)
    {
      case PCD_BMSTEP_1DDWORD:
      case PCD_BMSTEP_2DDWORD:
      case PCD_BMSTEP_4DDWORD:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_bi_reg->bmcon.all);
  tmp_val.b.bmstep = value;
  iti_writel (tmp_val.all, &iti->pcd_bi_reg->bmcon.all);

  return OK;
}

int
pcd_set_aqstart (itifg_t *iti, u_long value)
{
  if (value & (~PCD_AQSTART_MASK << PCD_AQSTART_SHIFT))
    {
      if ((value >> PCD_AQSTART_SHIFT) & PCD_AQSTART_MASK)
	iti->errno = -EITI_RANGE;
      else
	iti->errno = -EITI_MASK;
      return ERROR;
    }
  iti_writel ((value >> PCD_AQSTART_SHIFT) & PCD_AQSTART_MASK,
              &iti->pcd_bi_reg->aqstart);

  return OK;
}

int
pcd_set_intadr (itifg_t *iti, u_long value)
{
  if (value & (~PCD_INTADR_MASK << PCD_INTADR_SHIFT))
    {
      if ((value >> PCD_INTADR_SHIFT) & PCD_INTADR_MASK)
	iti->errno = -EITI_RANGE;
      else
	iti->errno = -EITI_MASK;
      return ERROR;
    }
  iti_writel ((value >> PCD_INTADR_SHIFT) & PCD_INTADR_MASK,
              &iti->pcd_bi_reg->intadr);

  return OK;
}

int
pcd_set_pad (itifg_t *iti, u_char value)
{
  pcd_bi_zoom_t tmp_val;

  switch (value)
    {
      case PCD_DISABLE:
      case PCD_PAD_MODE_SHIFT:
      case PCD_PAD_MODE_PAD16:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_bi_reg->zoom.all);
  tmp_val.b.pad = value;
  iti_writel (tmp_val.all, &iti->pcd_bi_reg->zoom.all);

  return OK;
}

int
pcd_set_segsize (itifg_t *iti, u_short value)
{
  if (value < PCD_SEGSIZE_MIN)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  value--;
  if (value & ~PCD_SEGSIZE_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_bi_reg->segsize);

  return OK;
}

int
pcd_set_octstart (itifg_t *iti, u_short value)
{
  if (value & ~PCD_OCTSTART_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_bi_reg->octstart);

  return OK;
}

int
pcd_set_swcount (itifg_t *iti, u_long value)
{
  if (value & ~PCD_SWCOUNT_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_bi_reg->swcount);

  return OK;
}

WRITE_HANDLER_BIT(ca,ctcon,lenenb)
WRITE_HANDLER_BIT(ca,ctcon,lensync)
WRITE_HANDLER_BIT(ca,ctcon,lenpol)
WRITE_HANDLER_BIT(ca,ctcon,fenenb)
WRITE_HANDLER_BIT(ca,ctcon,fenpol)
WRITE_HANDLER_BIT(ca,ctcon,multenb)
WRITE_HANDLER_BIT(ca,ctcon,multpol)
WRITE_HANDLER_BIT(ca,outcon,driveext)
WRITE_HANDLER_BIT(ca,outcon,useext)
WRITE_HANDLER_BIT(ca,outcon,restime)
WRITE_HANDLER_BIT(ca,outcon,respol)
WRITE_HANDLER_BIT(ca,outcon,cam3enb)
WRITE_HANDLER_BIT(ca,outcon,exsenb)
WRITE_HANDLER_BIT(ca,outcon,exspol)
WRITE_HANDLER_BIT(ca,outcon,prienb)
WRITE_HANDLER_BIT(ca,outcon,pripol)
WRITE_HANDLER_BIT(ca,outcon,cclkenb)
WRITE_HANDLER_BIT(ca,outcon,cclkpol)
WRITE_HANDLER_BIT(ca,cclkcon,syndat)
WRITE_HANDLER_BIT(ca,cclkcon,synclka)
WRITE_HANDLER_BIT(ca,cclkcon,synclkb)
WRITE_HANDLER_BIT(ca,cclkcon,synmuxa)
WRITE_HANDLER_BIT(ca,cclkcon,synmuxb)
WRITE_HANDLER_BIT(ca,lutcon0,lutacc)
WRITE_HANDLER_BIT(ca,trigcon,trig0pol)
WRITE_HANDLER_BIT(ca,trigcon,trig0gd)
WRITE_HANDLER_BIT(ca,trigcon,trig1pol)
WRITE_HANDLER_BIT(ca,trigcon,trig1gd)
WRITE_HANDLER_BIT(ca,intcon0,vbien)
WRITE_HANDLER_BIT(ca,intcon0,hbien)
WRITE_HANDLER_BIT(ca,intcon0,trig0ien)
WRITE_HANDLER_BIT(ca,intcon0,trig1ien)
WRITE_HANDLER_BIT(ca,intcon0,soaqien)
WRITE_HANDLER_BIT(ca,intcon0,eoaqien)
WRITE_HANDLER_BIT(ca,intcon0,pioien)
WRITE_HANDLER_BIT(ca,portcon,outstrb)
WRITE_HANDLER_BIT(ca,portcon,inregenb)
WRITE_HANDLER_BIT(ca,portcon,instrbpol)
WRITE_HANDLER_BIT(ca,portcon,inregclr)
WRITE_HANDLER_BIT(ca,portcon,inpintpol)

int
pcd_set_multmd (itifg_t *iti, u_char value)
{
  pcd_ca_ctcon_t tmp_val;

  switch (value)
    {
      case PCD_MULTI_FIELD:
      case PCD_MULTI_EOL:
      case PCD_MULTI_DATA_VALID:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->ctcon.all);
  tmp_val.b.multmd = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->ctcon.all);

  return OK;
}

int
pcd_set_exsmd (itifg_t *iti, u_char value)
{
  pcd_ca_outcon_t tmp_val;

  switch (value)
    {
      case PCD_EXSYNC_FREE_RUN:
      case PCD_EXSYNC_EXT_TRIG0:
      case PCD_EXSYNC_EXT_TRIG1:
      case PCD_EXSYNC_SW_TRIG0:
      case PCD_EXSYNC_SW_TRIG1:
      case PCD_EXSYNC_VB_TRIG:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->outcon.all);
  tmp_val.b.exsmd = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->outcon.all);

  return OK;
}

int
pcd_set_cammd (itifg_t *iti, u_char value)
{
  pcd_ca_outcon_t tmp_val;

  if (value > PCD_CAMMD_MAX)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->outcon.all);
  tmp_val.b.cammd = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->outcon.all);

  return OK;
}

int
pcd_set_exsclk (itifg_t *iti, u_char value)
{
  pcd_ca_exscon_t tmp_val;

  switch (value)
    {
      case PCD_EXSCLK_14_318_M:
      case PCD_EXSCLK_7_159_M:
      case PCD_EXSCLK_3_580_M:
      case PCD_EXSCLK_894_875_k:
      case PCD_EXSCLK_223_719_k:
      case PCD_EXSCLK_55_930_k:
      case PCD_EXSCLK_13_982_k:
      case PCD_EXSCLK_3_496_k:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  tmp_val.b.exsclk = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->exscon.all);

  return OK;
}

int
pcd_set_exssel (itifg_t *iti, u_char value)
{
  pcd_ca_exscon_t tmp_val;

  switch (value)
    {
      case PCD_EXSSEL_ONE_SHOT:
      case PCD_EXSSEL_STRT_TO_MID:
      case PCD_EXSSEL_MID_TO_STOP:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  tmp_val.b.exssel = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->exscon.all);

  return OK;
}

int
pcd_set_prisel (itifg_t *iti, u_char value)
{
  pcd_ca_exscon_t tmp_val;

  switch (value)
    {
      case PCD_PRISEL_ONE_SHOT:
      case PCD_PRISEL_STRT_TO_MID:
      case PCD_PRISEL_MID_TO_STOP:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  tmp_val.b.prisel = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->exscon.all);

  return OK;
}

int
pcd_set_emidpnt (itifg_t *iti, u_short value)
{
  pcd_ca_exscon_t tmp_val;

  if (value & (~PCD_EMIDPNT_MASK >> PCD_EMIDPNT_SHIFT))
    {
      if ((value << PCD_EMIDPNT_SHIFT) & PCD_EMIDPNT_MASK)
	iti->errno = -EITI_RANGE;
      else
	iti->errno = -EITI_MASK;
      return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  tmp_val.b.emidpnt = value >> PCD_EMIDPNT_SHIFT;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->exscon.all);

  return OK;
}

int
pcd_set_prihold (itifg_t *iti, u_char value)
{
  pcd_ca_exscon_t tmp_val;

  switch (value)
    {
      case PCD_PRIHOLD_LEN_STRT:
      case PCD_PRIHOLD_LEN_STOP:
      case PCD_PRIHOLD_NONE:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->exscon.all);
  tmp_val.b.prihold = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->exscon.all);

  return OK;
}

int
pcd_set_exstime (itifg_t *iti, u_long value)
{
  if (value < PCD_SEGSIZE_MIN)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  value -= 2;
  if (value & ~PCD_EXSTIME_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_ca_reg->exstime);

  return OK;
}

int
pcd_set_stmode (itifg_t *iti, u_char value)
{
  pcd_ca_acqcon_t tmp_val;

  switch (value)
    {
      case PCD_DISABLE:
      case PCD_TMODE_HWT0_SET:
      case PCD_TMODE_HWT0_RESET:
      case PCD_TMODE_HWT1_SET:
      case PCD_TMODE_HWT1_RESET:
      case PCD_TMODE_SWT0:
      case PCD_TMODE_SWT1:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  tmp_val.b.stmode = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->acqcon.all);

  return OK;
}

int
pcd_set_etmode (itifg_t *iti, u_char value)
{
  pcd_ca_acqcon_t tmp_val;

  switch (value)
    {
      case PCD_DISABLE:
      case PCD_TMODE_HWT0_SET:
      case PCD_TMODE_HWT0_RESET:
      case PCD_TMODE_HWT1_SET:
      case PCD_TMODE_HWT1_RESET:
      case PCD_TMODE_SWT0:
      case PCD_TMODE_SWT1:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  tmp_val.b.etmode = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->acqcon.all);

  return OK;
}

int
pcd_set_acqmd (itifg_t *iti, u_char value)
{
  pcd_ca_acqcon_t tmp_val;

  switch (value)
    {
      case PCD_FREEZE:
      case PCD_SNAP:
      case PCD_GRAB:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  tmp_val.b.acqmd = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->acqcon.all);

  return OK;
}

int
pcd_set_fldsel (itifg_t *iti, u_char value)
{
  pcd_ca_acqcon_t tmp_val;

  switch (value)
    {
      case PCD_NEXT_FIELD:
      case PCD_EVEN_FIELD:
      case PCD_ODD_FIELD:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  tmp_val.b.fldsel = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->acqcon.all);

  return OK;
}

int
pcd_set_fcnt (itifg_t *iti, u_char value)
{
  pcd_ca_acqcon_t tmp_val;
  
  if (value != PCD_FCNT_INFINITE)
    {
      if (value < PCD_FCNT_MIN || value > PCD_FCNT_MAX)
	{
	  iti->errno = -EITI_RANGE;
	  return ERROR;
	}
      value--;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->acqcon.all);
  tmp_val.b.fcnt = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->acqcon.all);

  return OK;
}

int
pcd_set_hoff (itifg_t *iti, u_short value)
{
  if (value & ~PCD_HOFF_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_ca_reg->hoff);

  return OK;
}

int
pcd_set_hact (itifg_t *iti, u_long value)
{
  if (value < PCD_HACT_MIN)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  value--;
  if (value & ~PCD_HACT_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_ca_reg->hact);

  return OK;
}

int
pcd_set_voff (itifg_t *iti, u_short value)
{
  if (value & ~PCD_VOFF_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_ca_reg->voff);

  return OK;
}

int
pcd_set_vact (itifg_t *iti, u_long value)
{
  if (value < PCD_VACT_MIN)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  value--;
  if (value & ~PCD_VACT_MASK)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  iti_writel (value, &iti->pcd_ca_reg->vact);

  return OK;
}

int
pcd_set_lut0psz (itifg_t *iti, u_char value)
{
  pcd_ca_lutcon0_t tmp_val;

  switch (value)
    {
      case PCD_PSIZE_8:
      case PCD_PSIZE_10:
      case PCD_PSIZE_12:
      case PCD_PSIZE_14:
      case PCD_PSIZE_16:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon0.all);
  tmp_val.b.lut0psz = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->lutcon0.all);

  return OK;
}

int
pcd_set_lut0pg (itifg_t *iti, u_char value)
{
  pcd_ca_lutcon0_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon0.all);
  tmp_val.b.lut0pg = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->lutcon0.all);

  return OK;
}

int
pcd_set_lut1psz (itifg_t *iti, u_char value)
{
  pcd_ca_lutcon1_t tmp_val;

  switch (value)
    {
      case PCD_PSIZE_8:
      case PCD_PSIZE_10:
      case PCD_PSIZE_12:
      case PCD_PSIZE_14:
      case PCD_PSIZE_16:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon1.all);
  tmp_val.b.lut1psz = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->lutcon1.all);

  return OK;
}

int
pcd_set_lut1pg (itifg_t *iti, u_char value)
{
  pcd_ca_lutcon1_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->lutcon1.all);
  tmp_val.b.lut1pg = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->lutcon1.all);

  return OK;
}

int
pcd_set_trig0src (itifg_t *iti, u_char value)
{
  pcd_ca_trigcon_t tmp_val;

  switch (value)
    {
      case PCD_TRIG_SRC_DIFF:
      case PCD_TRIG_SRC_TTL:
      case PCD_TRIG_SRC_OPTO:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigcon.all);
  tmp_val.b.trig0src = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->trigcon.all);

  return OK;
}

int
pcd_set_trig1src (itifg_t *iti, u_char value)
{
  pcd_ca_trigcon_t tmp_val;

  switch (value)
    {
      case PCD_TRIG_SRC_DIFF:
      case PCD_TRIG_SRC_TTL:
      case PCD_TRIG_SRC_OPTO:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigcon.all);
  tmp_val.b.trig1src = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->trigcon.all);

  return OK;
}

int
pcd_set_trigdiv (itifg_t *iti, u_short value)
{
  pcd_ca_trigdiv_t tmp_val;

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigdiv.all);
  tmp_val.b.trigdiv = --value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->trigdiv.all);

  return OK;
 }

int
pcd_set_divcon (itifg_t *iti, u_short value)
{
  pcd_ca_trigdiv_t tmp_val;

  switch (value)
    {
      case PCD_DIVCON_CLEAR:
      case PCD_DIVCON_TRIG0:
      case PCD_DIVCON_TRIG1:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->trigdiv.all);
  tmp_val.b.divcon = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->trigdiv.all);

  return OK;
}

int
pcd_set_pmidpnt (itifg_t *iti, u_short value)
{
  if (value & (~PCD_PMIDPNT_MASK << PCD_PMIDPNT_SHIFT))
    {
      if ((value >> PCD_PMIDPNT_SHIFT) & PCD_PMIDPNT_MASK)
	iti->errno = -EITI_RANGE;
      else
	iti->errno = -EITI_MASK;
      return ERROR;
    }
  iti_writel ((value >> PCD_PMIDPNT_SHIFT) & PCD_PMIDPNT_MASK,
              &iti->pcd_ca_reg->pmidpnt);

  return OK;
}

int
pcd_set_hdec (itifg_t *iti, u_char value)
{
  pcd_ca_decim_t tmp_val;

  switch (value)
    {
      case PCD_DEC_BY_1:
      case PCD_DEC_BY_2:
      case PCD_DEC_BY_4:
      case PCD_DEC_BY_8:
      case PCD_DEC_BY_16:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->decim.all);
  tmp_val.b.hdec = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->decim.all);

  return OK;
}

int
pcd_set_vdec (itifg_t *iti, u_char value)
{
  pcd_ca_decim_t tmp_val;

  switch (value)
    {
      case PCD_DEC_BY_1:
      case PCD_DEC_BY_2:
      case PCD_DEC_BY_4:
      case PCD_DEC_BY_8:
      case PCD_DEC_BY_16:
	break;
      default:
	iti->errno = -EITI_NOSUPPORT;
	return ERROR;
    }

  tmp_val.all = iti_readl (&iti->pcd_ca_reg->decim.all);
  tmp_val.b.vdec = value;
  iti_writel (tmp_val.all, &iti->pcd_ca_reg->decim.all);

  return OK;
}

int
pcd_set_outport (itifg_t *iti, u_char value)
{
  u_long tmp_val;

  tmp_val = iti_readl (&iti->pcd_ca_reg->outport);
  tmp_val = value;
  iti_writel (tmp_val, &iti->pcd_ca_reg->outport);

  return OK;
}

WRITE_HANDLER_BIT(fp,fpgacon,fpgaprg)

int
pcd_set_fpgadata (itifg_t *iti, u_char value)
{
  iti_writel (value, &iti->pcd_fp_reg->fpgadata);

  return OK;
}

/*
 * Special Clear Handler for PCDig Control Registers
 */

/*
 * Special Initialization Handler for PCDig Control Registers
 */

int
pcd_init_bmcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_bi_reg->bmcon.all);
  return OK;
}

int
pcd_init_bmlut (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_bi_reg->bmlut.all);
  return OK;
}

int
pcd_init_sginit (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_bi_reg->sgint);
  return OK;
}

int
pcd_init_zoom (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_bi_reg->zoom.all);
  return OK;
}

int
pcd_init_intcon1 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_bi_reg->intcon1.all);
  return OK;
}

int
pcd_init_acqcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->acqcon.all);
  return OK;
}

int
pcd_init_fifoinit (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->fifoinit);
  return OK;
}

int
pcd_init_intcon0 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->intcon0.all);
  return OK;
}

int
pcd_init_decim (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->decim.all);
  return OK;
}

int
pcd_init_portcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->portcon.all);
  return OK;
}


int
pcd_init_ctcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->ctcon.all);
  return OK;
}

int
pcd_init_outcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->outcon.all);
  return OK;
}

int
pcd_init_exscon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->exscon.all);
  return OK;
}

int
pcd_init_cclkcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->cclkcon.all);
  return OK;
}

int
pcd_init_lutcon0 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->lutcon0.all);
  return OK;
}

int
pcd_init_lutcon1 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->lutcon1.all);
  return OK;
}

int
pcd_init_trigcon (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->trigcon.all);
  return OK;
}

int
pcd_init_trigdiv (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->trigdiv.all);
  return OK;
}

/* 
 * Miscellanous functions for PCDig
 */
 
u_char
pcd_get_pixsz (itifg_t *iti)
{
  switch (pcd_get_sform (iti))
    {
    case PCD_SFORM_8BIT_ST:
    case PCD_SFORM_8BIT_DT:
    case PCD_SFORM_8BIT_QT:
      return 1;
    case PCD_SFORM_16BIT_ST:
    case PCD_SFORM_16BIT_DT:
      return 2;
    case PCD_SFORM_24BIT_PST:
      return 3; 
    case PCD_SFORM_32BIT_ST:
      return 4;
    }
  
  iti->errno = -EITI_NOSUPPORT;
  return ERROR;
}

/* trigger per software */
int
pcd_sw_trig0 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->swtrig0);
  return OK;
}

int
pcd_sw_trig1 (itifg_t *iti)
{
  iti_writel (PCD_RES_INIT, &iti->pcd_ca_reg->swtrig1);
  return OK;
}

#if (DEBUG_ITIFG & 4)
/* make register dump */
void
pcd_dump_regs (itifg_t *iti)
{
  int i;
  u_long *reg_ptr;
	
  reg_ptr = (u_long*)iti->pcd_bi_reg;
  ITI_PRINT2("  bi-regs dump:\n");
  for (i = 0; i < sizeof(pcd_bi_reg_name) / sizeof(pcd_bi_reg_name[0]);
       i++, reg_ptr++)
    if (iti_strcmp (pcd_bi_reg_name[i], "error  "))
      ITI_PRINT2A("    %s 0x%p %#08lx\n", pcd_bi_reg_name[i], reg_ptr,
		  iti_readl (reg_ptr));

  reg_ptr = (u_long*)iti->pcd_ca_reg;
  ITI_PRINT2("  ca-regs dump:\n");
  for (i = 0; i < sizeof(pcd_ca_reg_name) / sizeof(pcd_ca_reg_name[0]);
       i++, reg_ptr++)
    if (iti_strcmp (pcd_ca_reg_name[i], "error  "))
      ITI_PRINT2A("    %s 0x%p %#08lx\n", pcd_ca_reg_name[i], reg_ptr,
		   iti_readl (reg_ptr));

  reg_ptr = (u_long*)iti->pcd_fp_reg;
  ITI_PRINT2A("    fpgacon 0x%p %#08lx\n", reg_ptr, iti_readl (reg_ptr));

}
# endif

int
pcd_load_fpga (itifg_t *iti, caddr_t exo_data)
{
  int index, delay;

  /* start reconfiguration cycle */
  pcd_set_fpgaprg (iti, PCD_DISABLE);
  pcd_set_fpgaprg (iti, PCD_ENABLE);
  pcd_set_fpgaprg (iti, PCD_DISABLE);

  /* test for cycle initiated */
  if (pcd_get_fpgadone (iti))
    {
      iti_printw ("PC-DIG FPGA's not in config mode!\n");
      return -EIO;
    }
  for (delay = 0;; delay++)
    {
      if (pcd_get_fpgardy (iti))
	{
	  ITI_PRINT1A ("    'Ready after init config' delay is %dus\n", delay);
	  break;
	}
      else if (delay >= 150)	/* normal value approx. 77us */
	{
	  iti_printw ("PC-DIG FPGA's not ready for new data!\n");
	  return -EIO;
	}
      iti_udelay (1);
    }
  
  index = 0;
  /* write exo file to data port */
  while (index < PCD_CONFIG_BYTES) 
    {
      pcd_set_fpgadata (iti, exo_data[index++]);
      iti_udelay (1);
      if (!pcd_get_fpgardy (iti))
	{
	  iti_udelay (1);
	  if (!pcd_get_fpgardy (iti))
	    {
	      iti_printw ("PC-DIG FPGA's not ready for new data!\n");
	      return -EIO;
	    }
	}
    }
  
  ITI_PRINT1A("    %d Bytes loaded to PC-DIG FPGA's.\n", index);
  
  /* write at least 0xff to the port */
  pcd_set_fpgadata (iti, 0xFF);

  for (delay = 1;; delay++)
    {
      if (pcd_get_fpgadone (iti))
	{
	  iti->fpga_ok = TRUE;
	  ITI_PRINT1A("    'Ready after config done' delay is %dus\n", delay);
	  break;
	}
      else if (delay >= 3)	/* normal value approx. 1us */
	{
	  iti_printw ("PC-DIG FPGA's not ready for new data!\n");
	  return -EIO;
	}
      iti_udelay (1);
    }
  
  return OK;
}


/*
 * Calc/Write Function for Clock Generator
 * for Details on Operation see Data Sheet ICD2051
 * from Cypress Semiconductor Corporation
 */

int
pcd_calc_clkval (itifg_t *iti, double frequency, u_long *clkvalue)
{
  /* this is based on the original DOS/Windows software (C) by ITI */
  const double freq_range[16] =
  {
    42.5, 47.5, 53.5, 58.5, 62.5,   68.5,  69.0,
    82.0, 87.0, 92.0, 92.1, 105.0, 115.0, 120.0
  };

  u_long index = 0, mux = 0, minP = 0, minQ = 0;
  double PdivQ, P, Q, startQ, endQ, Err, minErr;

  if (frequency > 120.0)
    {
      iti->errno = -EITI_RANGE;
      return ERROR;
    }
  while (frequency < 40.0)
    {
      frequency *= 2;
      mux++;
      if (mux > 7)
	{
	  iti->errno = -EITI_RANGE;
	  return ERROR;
	}
    }
  while (frequency > freq_range[index])
    index++;
  
  PdivQ = frequency / (2.0 * PCD_CLKVAL_REFCLOCK);
  startQ = (double)((u_long)(PCD_CLKVAL_REFCLOCK) + 1);
  if (startQ < 3.0) startQ=3.0;
  endQ = (double)((u_long)(PCD_CLKVAL_REFCLOCK / 0.2));
  if (endQ > 129.0) endQ = 129.0;

  minErr = 100;
  for (Q = startQ; Q < endQ; Q += 1.0)
    {
    P = Q * PdivQ;
    if ((double)(P - (u_long)P)  > 0.5)
      P = (double)((u_long)P + 1);
    else 				   
      P = (double)((u_long)P);
    Err = PdivQ - (P / Q);
    if (Err < 0)
      Err = -Err;
    if (Err < minErr && P >= 4 && P <= 130) {
      minErr = Err;
      minP = (u_long)P;
      minQ = (u_long)Q;
    }
  }
  minP -= 3;
  minQ -= 2;

  *clkvalue = (index << 18) | (minP << 11) | (1 << 10) | (mux << 7) | minQ;

  return OK;
}

int
pcd_write_clkval (itifg_t *iti, char device, u_long clkvalue)
{
  int error, count;

  if (clkvalue & ~PCD_CLKVAL_MASK)
    {
      iti_printwa ("error in pv_write_clkval (%#0lx > %#0x).\n",
		   clkvalue, PCD_CLKVAL_MASK);
      return ERROR;
    }
  
  switch (device)
    {
    case 'A':
      if ((error = pcd_set_synclka (iti, CR_DISABLE)))
	return error;
      break;
    case 'B':
      if ((error = pcd_set_synclkb (iti, CR_DISABLE)))
	return error;
      break;
    default:
      iti->errno = -EITI_NOSUPPORT;
      return ERROR;
    }
  ITI_PRINT2A("    WRITE %c - %#0lx:\n", device, clkvalue);

  for (count = 0; count < PCD_CLKVAL_SHIFT; count++)
    {
      if ((error = pcd_set_syndat (iti, clkvalue & 1)))
	return error;
      PCD_PRINTR("%ld", clkvalue & 1);
      clkvalue >>= 1;
      switch (device)
	{
	case 'A':
	  if ((error = pcd_set_synclka (iti, CR_ENABLE)))
	    return error;
	  if ((error = pcd_set_synclka (iti, CR_DISABLE)))
	    return error;
	  break;
	case 'B':
	  if ((error = pcd_set_synclkb (iti, CR_ENABLE)))
	    return error;
	  if ((error = pcd_set_synclkb (iti, CR_DISABLE)))
	    return error;
	  break;
	default:
	  iti->errno = -EITI_NOSUPPORT;
	  return ERROR;
	}
      PCD_PRINTR("|");
    }
  PCD_PRINTR("\n");
  
  return OK;
}

/* 'fake' functions for the virtual cr switch */

u_char
pcd_get_fldstat (itifg_t *iti)
{
  /* check the MULTI input with respect to the MULTI polarity */
  if (pcd_get_multpol == PCD_MULTI_ACT_LOW)
    {
    if (pcd_get_cammult == LOW)
      return CR_IN_EVEN_FIELD;
    else
      return CR_IN_ODD_FIELD;
    }
  else
    {
    if (pcd_get_cammult (iti) == HIGH)
      return CR_IN_EVEN_FIELD;
    else
      return CR_IN_ODD_FIELD;
    }
  
  /* NOTREACHED */
  return CR_IN_EVEN_FIELD;
}

u_char
pcd_get_vbstat (itifg_t *iti)
{
  /* check the CAMFEN input with respect to the CAMFEN polarity */
  if (pcd_get_fenpol == PCD_FEN_ACT_LOW)
    {
    if (pcd_get_camfen == LOW)
      return CR_OUT_VB;
    else
      return CR_IN_VB;
    }
  else
    {
    if (pcd_get_camfen (iti) == HIGH)
      return CR_OUT_VB;
    else
      return CR_IN_VB;
    }
  
  /* NOTREACHED */
  return CR_IN_VB;
}

u_char
pcd_get_smstat (itifg_t *iti)
{
  /* check FLDSEL field entry for next/even-odd value */
  switch (pcd_get_fldsel (iti))
    {
    case CR_EVEN_FIELD:
    case CR_ODD_FIELD:
      return CR_ILACE;
    }
  
  return CR_NON_ILACE;
}

u_char
pcd_get_tmode (itifg_t *iti)
{
  /* get the start of trigger field entry (Hardware Trigger 0 asserted) */
  if (pcd_get_stmode (iti) == PCD_TMODE_HWT0_SET)
    return CR_ENABLE;

  return CR_DISABLE;
}

int
pcd_set_tmode (itifg_t *iti, u_char value)
{

  /* set the start of trigger field entry (Hardware Trigger 0 asserted) */
  if (value == CR_ENABLE)
    return pcd_set_stmode (iti, PCD_TMODE_HWT0_SET);
  
  return OK;
}

/* FIXME */
int
pcd_update_exsync (itifg_t *iti)
{
  int pcdig, board;
  
  /* find first board */
  board = 0;
  while (itifg[board] != NULL && itifg[board] != iti)
    board++;
  
  for (pcdig = board; pcdig >= 0; pcdig--)
    if (itifg[pcdig]->device_id == ITI_PCDIG &&
	pcd_get_driveext (itifg[pcdig]) == PCD_ENABLE)
      {
	pcd_set_exsenb (itifg[pcdig], PCD_DISABLE);
	pcd_set_exsclk (itifg[pcdig], new_exsclk);
	pcd_set_exstime (itifg[pcdig], new_exstime);
	pcd_set_emidpnt (itifg[pcdig],
			 (new_midpnt / (1 << PCD_EMIDPNT_SHIFT)) *
			 (1 << PCD_EMIDPNT_SHIFT));
	pcd_set_pmidpnt (itifg[pcdig],
			 (new_midpnt / (1 << PCD_PMIDPNT_SHIFT)) *
			 (1 << PCD_PMIDPNT_SHIFT));
	pcd_set_exsenb (itifg[pcdig], PCD_ENABLE);
	return OK;
      }

  return ERROR;
}

